"use strict";
/* eslint-disable import/extensions */
/* eslint-disable unicorn/prefer-spread */
/* eslint-disable unicorn/prefer-code-point */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.decode = exports.decodeAsyncGenerator = exports.decodeGenerator = exports.endsWithIncompleteUtfPairSurrogate = exports.decodeToken = exports.encode = exports.isWithinTokenLimit = exports.encodeGenerator = exports.bpe = void 0;
const bpeRanks_js_1 = __importDefault(require("./data/bpeRanks.js"));
const encoder_js_1 = __importDefault(require("./data/encoder.js"));
const utils_js_1 = require("./utils.js");
const encoder = encoder_js_1.default;
const bpeRanks = bpeRanks_js_1.default;
const decoder = (0, utils_js_1.dictZip)(Object.values(encoder), Object.keys(encoder));
const textEncoder = new TextEncoder();
const textDecoder = new TextDecoder();
// eslint-disable-next-line no-magic-numbers
const UPPER_BYTE_VALUE = 2 ** 8;
function createByteToUnicodeMap() {
    const asciiRange = (0, utils_js_1.range)((0, utils_js_1.getCharCode)('!'), (0, utils_js_1.getCharCode)('~') + 1);
    const latin1Range1 = (0, utils_js_1.range)((0, utils_js_1.getCharCode)('¡'), (0, utils_js_1.getCharCode)('¬') + 1);
    const latin1Range2 = (0, utils_js_1.range)((0, utils_js_1.getCharCode)('®'), (0, utils_js_1.getCharCode)('ÿ') + 1);
    const initialCodePoints = [...asciiRange, ...latin1Range1, ...latin1Range2];
    const mappedCodePoints = [...initialCodePoints];
    let newCodePointOffset = 0;
    for (let byteValue = 0; byteValue < UPPER_BYTE_VALUE; byteValue++) {
        if (!initialCodePoints.includes(byteValue)) {
            initialCodePoints.push(byteValue);
            mappedCodePoints.push(UPPER_BYTE_VALUE + newCodePointOffset);
            newCodePointOffset += 1;
        }
    }
    const unicodeChars = mappedCodePoints.map(utils_js_1.getCharFromCode);
    return new Map(initialCodePoints.map((x, i) => [x, unicodeChars[i]]));
}
const pat = /'s|'t|'re|'ve|'m|'ll|'d| ?\p{L}+| ?\p{N}+| ?[^\s\p{L}\p{N}]+|\s+(?!\S)|\s+/gu;
const byteEncoder = createByteToUnicodeMap();
const byteDecoder = new Map([...byteEncoder].map(([k, v]) => [v, k]));
function bpe(token, cache = new Map()) {
    if (cache.has(token)) {
        return cache.get(token);
    }
    let word = (0, utils_js_1.splitToken)(token);
    let pairs = (0, utils_js_1.getPairs)(word);
    if (pairs.length === 0) {
        return token;
    }
    // eslint-disable-next-line no-constant-condition
    while (true) {
        const minPairs = new Map();
        pairs.forEach((pair) => {
            const rank = bpeRanks[pair.join(',')];
            minPairs.set(
            // eslint-disable-next-line no-magic-numbers
            typeof rank === 'undefined' || Number.isNaN(rank) ? 10e10 : rank, pair);
        });
        const keys = [...minPairs.keys()];
        const bigram = minPairs.get(Math.min(...keys));
        if (!bigram || !(bigram.join(',') in bpeRanks)) {
            break;
        }
        const [first, second] = bigram;
        let newWord = [];
        let i = 0;
        while (i < word.length) {
            const j = word.indexOf(first, i);
            if (j === -1) {
                newWord = newWord.concat(word.slice(i));
                break;
            }
            newWord = newWord.concat(word.slice(i, j));
            i = j;
            if (word[i] === first && i < word.length - 1 && word[i + 1] === second) {
                newWord.push(first + second);
                i += 2;
            }
            else {
                newWord.push(word[i]);
                i += 1;
            }
        }
        word = newWord;
        if (word.length === 1) {
            break;
        }
        else {
            pairs = (0, utils_js_1.getPairs)(word);
        }
    }
    const result = word.join(' ');
    cache.set(token, result);
    return result;
}
exports.bpe = bpe;
function* encodeGenerator(text, cache = new Map()) {
    for (let [token] of text.matchAll(pat)) {
        token = [...textEncoder.encode(token)]
            .map((x) => byteEncoder.get(x) ?? '')
            .join('');
        const newTokens = bpe(token, cache)
            .split(' ')
            .map((x) => encoder[x]);
        yield newTokens;
    }
}
exports.encodeGenerator = encodeGenerator;
/**
 * @returns {false | number} false if token limit is exceeded, otherwise the number of tokens
 */
function isWithinTokenLimit(text, tokenLimit, cache = new Map()) {
    const tokenGenerator = encodeGenerator(text, cache);
    let count = 0;
    for (const tokens of tokenGenerator) {
        count += tokens.length;
        if (count > tokenLimit) {
            return false;
        }
    }
    return count;
}
exports.isWithinTokenLimit = isWithinTokenLimit;
function encode(text, cache = new Map()) {
    return [...encodeGenerator(text, cache)].flat(1);
}
exports.encode = encode;
function decodeToken(token) {
    const decodedToken = decoder[token];
    if (typeof decodedToken === 'undefined') {
        return '';
    }
    const decodedBytes = (0, utils_js_1.splitToken)(decodedToken).map((x) => byteDecoder.get(x));
    return textDecoder.decode(new Uint8Array(decodedBytes), {
        stream: true,
    });
}
exports.decodeToken = decodeToken;
const HIGH_SURROGATE_START = 55296;
const HIGH_SURROGATE_END = 56319;
function endsWithIncompleteUtfPairSurrogate(string) {
    if (string.length === 0)
        return false;
    // Check if the last character is a high surrogate
    const lastCharCode = string.charCodeAt(string.length - 1);
    return (lastCharCode >= HIGH_SURROGATE_START && lastCharCode <= HIGH_SURROGATE_END);
}
exports.endsWithIncompleteUtfPairSurrogate = endsWithIncompleteUtfPairSurrogate;
function* decodeGenerator(tokens) {
    let buffer = '';
    for (const token of tokens) {
        buffer += decodeToken(token);
        if (buffer.length === 0 || endsWithIncompleteUtfPairSurrogate(buffer)) {
            // Keep the high surrogate in the buffer and continue with the next token
            // eslint-disable-next-line no-continue
            continue;
        }
        else {
            yield buffer;
            // reset buffer
            buffer = '';
        }
    }
    // Yield any remaining characters in the buffer
    if (buffer.length > 0) {
        yield buffer;
    }
}
exports.decodeGenerator = decodeGenerator;
/**
 * Decode tokens asynchronously and yield the decoded strings, one by one.
 * Will not yield for tokens that include a high surrogate, but wait for the next token.
 * @param {AsyncIterable<number>} tokens
 * @returns {AsyncGenerator<string, void, undefined>}
 */
async function* decodeAsyncGenerator(tokens) {
    let buffer = '';
    for await (const token of tokens) {
        buffer += decodeToken(token);
        if (buffer.length === 0 || endsWithIncompleteUtfPairSurrogate(buffer)) {
            // Keep the high surrogate in the buffer and continue with the next token
            // eslint-disable-next-line no-continue
            continue;
        }
        else {
            yield buffer;
            // reset buffer
            buffer = '';
        }
    }
    // Yield any remaining characters in the buffer
    if (buffer.length > 0) {
        yield buffer;
    }
}
exports.decodeAsyncGenerator = decodeAsyncGenerator;
function decode(tokens) {
    return [...decodeGenerator(tokens)].join('');
}
exports.decode = decode;
const gptEncoder = {
    encode,
    decode,
    encodeGenerator,
    decodeGenerator,
    decodeAsyncGenerator,
    isWithinTokenLimit,
    bpe,
};
// used by the UMD export
// eslint-disable-next-line import/no-default-export
exports.default = gptEncoder;
//# sourceMappingURL=main.js.map