"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable compat/compat */
const main_1 = require("./main");
describe('basic functionality', () => {
    it('empty string', () => {
        const str = '';
        expect((0, main_1.encode)(str)).toEqual([]);
        expect((0, main_1.decode)((0, main_1.encode)(str))).toEqual(str);
        expect((0, main_1.isWithinTokenLimit)(str, 0)).toBe(0);
        expect((0, main_1.isWithinTokenLimit)(str, 3)).toBe(0);
    });
    it('space', () => {
        const str = ' ';
        expect((0, main_1.encode)(str)).toEqual([220]);
        expect((0, main_1.decode)((0, main_1.encode)(str))).toEqual(str);
        expect((0, main_1.isWithinTokenLimit)(str, 3)).toBe(1);
        expect((0, main_1.isWithinTokenLimit)(str, 0)).toBe(false);
    });
    it('tab', () => {
        const str = '\t';
        expect((0, main_1.encode)(str)).toEqual([197]);
        expect((0, main_1.decode)((0, main_1.encode)(str))).toEqual(str);
    });
    it('simple text', () => {
        const str = 'This is some text';
        expect((0, main_1.encode)(str)).toEqual([1212, 318, 617, 2420]);
        expect((0, main_1.decode)((0, main_1.encode)(str))).toEqual(str);
        expect((0, main_1.isWithinTokenLimit)(str, 3)).toBe(false);
        expect((0, main_1.isWithinTokenLimit)(str, 5)).toBe(4);
    });
    it('multi-token word', () => {
        const str = 'indivisible';
        expect((0, main_1.encode)(str)).toEqual([521, 452, 12843]);
        expect((0, main_1.decode)((0, main_1.encode)(str))).toEqual(str);
        expect((0, main_1.isWithinTokenLimit)(str, 3)).toBe(3);
    });
    const helloWorldTokens = [31373, 50169, 233, 995, 12520, 234, 235];
    it('emojis', () => {
        const str = 'hello 👋 world 🌍';
        expect((0, main_1.encode)(str)).toEqual(helloWorldTokens);
        expect((0, main_1.decode)((0, main_1.encode)(str))).toEqual(str);
        expect((0, main_1.isWithinTokenLimit)(str, 4)).toBe(false);
        expect((0, main_1.isWithinTokenLimit)(str, 400)).toBe(7);
    });
    it('decode token-by-token via generator', () => {
        const generator = (0, main_1.decodeGenerator)(helloWorldTokens);
        expect(generator.next().value).toBe('hello');
        expect(generator.next().value).toBe(' ');
        expect(generator.next().value).toBe('👋');
        expect(generator.next().value).toBe(' world');
        expect(generator.next().value).toBe(' ');
        expect(generator.next().value).toBe('🌍');
    });
    async function* getHelloWorldTokensAsync() {
        for (const token of helloWorldTokens) {
            // eslint-disable-next-line no-await-in-loop
            yield await Promise.resolve(token);
        }
    }
    it('decode token-by-token via async generator', async () => {
        const generator = (0, main_1.decodeAsyncGenerator)(getHelloWorldTokensAsync());
        const decoded = ['hello', ' ', '👋', ' world', ' ', '🌍'];
        for await (const value of generator) {
            expect(value).toEqual(decoded.shift());
        }
    });
    it('properties of Object', () => {
        const str = 'toString constructor hasOwnProperty valueOf';
        expect((0, main_1.encode)(str)).toEqual([
            1462, 10100, 23772, 468, 23858, 21746, 1988, 5189,
        ]);
        expect((0, main_1.decode)((0, main_1.encode)(str))).toEqual(str);
    });
    it('text with commas', () => {
        const str = 'hello, I am a text, and I have commas. a,b,c';
        expect((0, main_1.decode)((0, main_1.encode)(str))).toEqual(str);
        expect((0, main_1.encode)(str)).toStrictEqual([
            31373, 11, 314, 716, 257, 2420, 11, 290, 314, 423, 725, 292, 13, 257,
            11, 65, 11, 66,
        ]);
        expect((0, main_1.isWithinTokenLimit)(str, 15)).toBe(false);
        expect((0, main_1.isWithinTokenLimit)(str, 300)).toBe(18);
    });
});
//# sourceMappingURL=main.test.js.map